﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Problems.Scheduling.Encodings.JobSequenceMatrix;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;


namespace HeuristicLab.Problems.Scheduling.Encodings.JobSequenceMatrix {
  [Item("Job Sequencing Matrix Creator", "Creator class used to create Job Sequencing Matrix solutions for standard JobShop scheduling problems.")]
  [StorableClass]
  public class JSMRandomCreator : JSSPCreator<JSMEncoding>, IStochasticOperator {
    [StorableConstructor]
    protected JSMRandomCreator(bool deserializing) : base(deserializing) { }
    protected JSMRandomCreator(JSMRandomCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSMRandomCreator(this, cloner);
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public IRandom Random {
      get { return RandomParameter.ActualValue; }
    }

    public JSMRandomCreator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
    }


    protected override JSMEncoding CreateSolution() {
      JSMEncoding solution = new JSMEncoding();
      IntValue nrOfJobs = new IntValue (Jobs.Count);
      IntValue nrOfResources = new IntValue (Jobs[0].Tasks.Count);

      for (int i = 0; i < nrOfResources.Value; i++) {
        solution.JobSequenceMatrix.Add(new Permutation(PermutationTypes.Absolute, nrOfJobs.Value, Random));
      }

      return solution;
    }
  }
}
