﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Problems.Scheduling.Interfaces;
using HeuristicLab.Encodings.IntegerVectorEncoding;

namespace HeuristicLab.Problems.Scheduling.Encodings.PriorityRulesVector.Crossovers {
  [Item("PRV SinglePoint Crossover", "Represents a crossover operation swapping sequences of the parents to generate offspring.")]
  [StorableClass]
  public class PRVSinglePointCrossover : PRVCrossover {
    [StorableConstructor]
    protected PRVSinglePointCrossover(bool deserializing) : base(deserializing) { }
    protected PRVSinglePointCrossover(PRVSinglePointCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PRVSinglePointCrossover(this, cloner);
    }

    public PRVSinglePointCrossover() : base() { }

    public static IJSSPEncoding Cross(IRandom random, PRVEncoding parent1, PRVEncoding parent2) {
      return new PRVEncoding(SinglePointCrossover.Apply(random, parent1, parent2), parent1.NrOfRules);
    }

    public override IJSSPEncoding Crossover(IRandom random, PRVEncoding parent1, PRVEncoding parent2) {
      return Cross(random, parent1, parent2);
    } 
  }
}
