﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Parameters;
using HeuristicLab.Encodings.PermutationEncoding;
using System.ComponentModel;

namespace HeuristicLab.Problems.Scheduling {
  [Item("Schedule Class", "Represents the general solution for scheduling problems.")]
  [StorableClass]
  public class Schedule : NamedItem{
    [StorableConstructor]
    protected Schedule(bool deserializing) : base(deserializing) { }
    protected Schedule(Schedule original, Cloner cloner)
      : base(original, cloner) {
        this.Resources = cloner.Clone (original.Resources);
        this.Quality = cloner.Clone(original.Quality);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Schedule(this, cloner);
    }

    [Storable]
    public ItemList<Resource> Resources;

    [Storable]
    public DoubleValue Quality {
      get;
      set;
    }

    public Schedule(IntValue nrOfResources) {
      Resources = new ItemList<Resource>();
      Quality = new DoubleValue();
      for (int i = 0; i < nrOfResources.Value; i++) {
        Resources.Add(new Resource(new IntValue (i)));
      }
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[ ");
      foreach (Resource r in Resources) {
        sb.Append(r.ToString () + " \n");
      }
      sb.Append("]");
      return sb.ToString();
    }
  }
}
